from typing import List

from src.models.fixture_stats import FixtureStats
from src.repositories.fixture_stats_repository import FixtureStatsRepository
from src.services.fixture_service import FixtureService


class FixtureStatsService:
    def __init__(self, db_name):
        self.fixture_stats_repository = FixtureStatsRepository(db_name)
        self.fixture_service = FixtureService()

    def upsert_many_fixture_stats(self, fixture_stats: List[dict]):
        self.fixture_stats_repository.upsert_many_fixture_stats(fixture_stats)

    def get_fixture_stats(self, fixture_id: str):
        response = self.fixture_stats_repository.get_fixture_stats(fixture_id)

        if not response.get('data', []):
            raise ValueError(f'No fixture stats found for fixture {fixture_id}')

        fixture_stats = FixtureStats.from_dict(response)
        return fixture_stats

    def get_fixture_stats_over_season(self, team_id: str, league_id: str, season: str):
        fixture_ids = self.fixture_service.get_fixture_ids(team_id, league_id, season)
        fixture_stats = self.fixture_stats_repository.get_team_fixture_stats(fixture_ids)
        fixture_stats = [FixtureStats.from_dict(fixture_stat) for fixture_stat in fixture_stats]
        return fixture_stats



from datetime import datetime

from src.models.league import League
from src.models.team import Team


class TeamStats:
    def __init__(
            self,
            season,
            date,
            league,
            team,
            form,
            fixtures_played,
            fixtures_wins,
            fixtures_draws,
            fixtures_loses,

            goals_for,
            goals_against,

            biggest_streak,
            biggest_wins,
            biggest_loses,
            biggest_goals,

            clean_sheet,
            failed_to_score
    ):
        self.team = team
        self.league = league
        self.form = form
        self.fixtures_played = fixtures_played
        self.fixtures_wins = fixtures_wins
        self.fixtures_draws = fixtures_draws
        self.fixtures_loses = fixtures_loses
        self.goals_for = goals_for
        self.goals_against = goals_against
        self.biggest_streak = biggest_streak
        self.biggest_wins = biggest_wins
        self.biggest_loses = biggest_loses
        self.biggest_goals = biggest_goals
        self.clean_sheet = clean_sheet
        self.failed_to_score = failed_to_score

    @classmethod
    def from_dict(cls, response):
        date = response['parameters']['date']
        season = response['parameters']['season']
        data = response['response']

        return cls(
            season=season,
            date=datetime.strptime(date, '%Y-%m-%d'),
            team=Team(
                team_id=data['team']['id'],
                name=data['team']['name'],
            ),
            league=League(
                league_id=data['league']['id'],
                name=data['league']['name']
            ),
            form=data['form'],
            fixtures_played=data['fixtures']['played'],
            fixtures_wins=data['fixtures']['wins'],
            fixtures_draws=data['fixtures']['draws'],
            fixtures_loses=data['fixtures']['loses'],
            goals_for=data['goals']['for'],
            goals_against=data['goals']['against'],
            biggest_streak=data['biggest']['streak'],
            biggest_wins=data['biggest']['wins'],
            biggest_loses=data['biggest']['loses'],
            biggest_goals=data['biggest']['goals'],
            clean_sheet=data['clean_sheet'],
            failed_to_score=data['failed_to_score']
        )



from src.models.team import Team


class XGoals:
    def __init__(
            self,
            fixture_id: str,
            date,

            home_team: Team,
            away_team: Team,

            home_xg,
            away_xg,

            home_goals,
            away_goals
    ):
        self.fixture_id = fixture_id
        self.date = date

        self.home_team = home_team
        self.away_team = away_team

        self.home_xg = home_xg
        self.away_xg = away_xg

        self.home_goals = home_goals
        self.away_goals = away_goals

    @classmethod
    def from_dict(cls, data):
        return cls(
            fixture_id=data['fixture']['id'],
            date=data['date'],

            home_team=Team(
                team_id=data['teams']['home']['id'],
                name=data['teams']['home']['name'],
            ),
            away_team=Team(
                team_id=data['teams']['away']['id'],
                name=data['teams']['away']['name'],
            ),

            home_xg=data['xg']['home'],
            away_xg=data['xg']['away'],

            home_goals=data['goals']['home'],
            away_goals=data['goals']['away']
        )



from src.models.team import Team
from src.models.team_fixture_stats import TeamFixtureStats


class FixtureStats:
    def __init__(
            self,
            fixture_id,
            home_fixture_stats,
            away_fixture_stats,
    ):
        self.fixture_id = fixture_id
        self.home_fixture_stats = home_fixture_stats
        self.away_fixture_stats = away_fixture_stats

    @classmethod
    def from_dict(cls, response):
        fixture_id = response['parameters']['fixture']

        home_fixture_stats = TeamFixtureStats.from_dict(response['data'][0])
        away_fixture_stats = TeamFixtureStats.from_dict(response['data'][1])

        return cls(
            fixture_id=fixture_id,
            home_fixture_stats=home_fixture_stats,
            away_fixture_stats=away_fixture_stats,
        )



import logging
from typing import List

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

logger = logging.getLogger(__name__)


class InjuryRepository(MongoDBBaseRepository):
    DB_NAME = 'api_football'

    def __init__(self):
        super().__init__(self.DB_NAME)

    def upsert_many_injuries(self, injuries):
        self.bulk_upsert_documents('injuries', injuries)
        logger.info('Upsert fixture stats data')

    def get_injuries(self, fixture_id: str):
        query = {'parameters.fixture': fixture_id}
        injuries = self.find_document('injuries', query)
        logger.info(f'Fetching injuries for fixture {fixture_id}')
        return injuries

    def get_team_injuries(self, fixture_ids: List[str]):
        query = {'parameters.fixture': {'$in': fixture_ids}}
        injuries = self.find_documents('injuries', query)
        logger.info(f'Fetching lineups for fixtures {fixture_ids}')
        return injuries



import logging

from src.models.lineups import Lineups
from src.repositories.lineups_repository import LineupsRepository
from src.services.fixture_service import FixtureService

LOGGER = logging.getLogger(__name__)


class LineupsService:
    def __init__(self):
        self.lineups_repository = LineupsRepository()
        self.fixture_service = FixtureService()

    def upsert_many_lineups(self, fixture_injuries):
        self.lineups_repository.bulk_upsert_documents('lineups', fixture_injuries)
        LOGGER.info('Upserted fixture stats data')

    def get_lineups(self, fixture_id: str):
        response = self.lineups_repository.get_lineups(fixture_id)

        if not response.get('data', []):
            return None

        lineups = Lineups.from_dict(response)
        return lineups

    def get_team_lineups(self, team_id: str, league_id: str, season: str):
        fixture_ids = self.fixture_service.get_fixture_ids(team_id, league_id, season)
        lineups = self.lineups_repository.get_many_lineups(fixture_ids)
        lineups = [Lineups.from_dict(lineup) for lineup in lineups]

        return lineups

    def get_league_lineups(self, league_id: str, season: str):
        fixture_ids = self.fixture_service.get_league_fixture_ids(league_id, season)
        lineups = self.lineups_repository.get_many_lineups(fixture_ids)
        lineups = [Lineups.from_dict(lineup) for lineup in lineups]

        return lineups



import logging
from typing import List

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

logger = logging.getLogger(__name__)


class TeamStatsRepository(MongoDBBaseRepository):
    DB_NAME = 'api_football'

    def __init__(self):
        super().__init__(self.DB_NAME)

    def upsert_many_team_stats(self, team_stats: List[dict]):
        self.bulk_upsert_documents('teams_stats', team_stats)
        logger.info('Upserted team stats data')

    def get_team_stats(self, team_id: str, league_id: str, season: str):
        query = {'parameters.team': team_id, 'parameters.league': league_id, 'parameters.season': season}
        team_stats = self.find_document('teams_stats', query)
        logger.info(f'Fetching stats for team {team_id} for league {league_id} for season {season}')
        return team_stats



from src.models.team import Team


class TeamLineups:
    def __init__(
            self,
            team,
            startXI,
            substitutes,
            formation
    ):
        self.team = team
        self.startXI = startXI
        self.substitutes = substitutes
        self.formation = formation

    @classmethod
    def from_dict(cls, data):
        return cls(
            team=Team(
                team_id=data['team']['id'],
                name=data['team']['name']
            ),
            startXI=data['startXI'],
            substitutes=data['substitutes'],
            formation=data['formation']
        )



import logging
from typing import List, Union

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

LOGGER = logging.getLogger(__name__)


class XGoalsRepository(MongoDBBaseRepository):

    def __init__(self, db_name):
        super().__init__(db_name)

    def upsert_many_fixture_xg(self, xg):
        self.bulk_upsert_documents('xgoals', xg)
        LOGGER.info('Upserted fixture xgoals data')

    def get_xgoals(self, fixture_id: str):
        query = {'parameters.fixture': fixture_id}
        xg = self.find_document('xgoals', query)
        LOGGER.info(f'Fetching xgoals for fixture {fixture_id}')
        return xg

    def get_many_xgoals(self, fixture_ids: List[str]):
        query = {'parameters.fixture': {'$in': fixture_ids}}
        xg = self.find_documents('xgoals', query)
        LOGGER.info(f'Fetching fbref xgoals for fixtures {fixture_ids}')
        return xg


import logging
from typing import List

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

logger = logging.getLogger(__name__)


class FixtureRepository(MongoDBBaseRepository):
    DB_NAME = 'api_football'

    def __init__(self):
        super().__init__(self.DB_NAME)

    def upsert_many_fixtures(self, fixtures: List[dict]):
        self.bulk_upsert_documents('fixtures', fixtures)
        logger.info('Upsert fixtures data')

    def get_team_fixtures(self, team_id: int, league_id: int, season: int) -> dict:
        query = {'parameters.team': team_id, 'parameters.league': league_id, 'parameters.season': season}
        fixtures = self.find_document('fixtures', query)
        logger.info(f'Fetching fixtures for team {team_id} for league {league_id} for season {season}')
        return fixtures

    def get_league_fixtures(self, league_id: int, season: int) -> List[dict]:
        query = {'parameters.league': league_id, 'parameters.season': season}
        fixtures = self.find_documents('fixtures', query)
        logger.info(f'Fetching fixtures for league {league_id} for season {season}')
        return fixtures

    def upsert_many_rounds(self, league_rounds):
        self.bulk_upsert_documents('league_rounds', league_rounds)
        logger.info('Upsert league rounds data')

    def get_rounds(self, league_id: int, season: int) -> dict:
        query = {'parameters.league': league_id, 'parameters.season': season}
        league_rounds = self.find_document('league_rounds', query)
        logger.info(f'Fetching league rounds for league {league_id} | season {season}')
        return league_rounds



import logging
from typing import List

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

logger = logging.getLogger(__name__)


class FixtureStatsRepository(MongoDBBaseRepository):
    def __init__(self, db_name):
        super().__init__(db_name)

    def upsert_many_fixture_stats(self, fixture_stats):
        self.bulk_upsert_documents('fixtures_stats', fixture_stats)
        logger.info('Upserted fixture stats data')

    def get_fixture_stats(self, fixture_id: str):
        query = {'parameters.fixture': fixture_id}
        stats = self.find_document('fixtures_stats', query)
        logger.info(f'Fetching stats for fixture {fixture_id}')
        return stats

    def get_team_fixture_stats(self, fixture_ids: List[str]):
        query = {'parameters.fixture': {'$in': fixture_ids}}
        fixture_stats = self.find_documents('fixtures_stats', query)
        logger.info(f'Fetching lineups for fixtures {fixture_ids}')
        return fixture_stats



import logging
from typing import List, Union

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

LOGGER = logging.getLogger(__name__)


class TeamRepository(MongoDBBaseRepository):
    DB_NAME = 'api_football'

    def __init__(self):
        super().__init__(self.DB_NAME)

    def upsert_many_teams(self, teams):
        self.bulk_upsert_documents('teams', teams)
        LOGGER.info('Upserted teams data')

    def get_teams(self, league_id: str, season: str):
        query = {'parameters.league': league_id, 'parameters.season': season}
        teams = self.find_document('teams', query)
        LOGGER.info(f'Fetching teams for league {league_id} for season {season}')
        return teams

    def get_many_teams(self, league_id: int, seasons: Union[int, List[int]]) -> List[str]:
        seasons = [seasons] if isinstance(seasons, int) else seasons
        query = {'parameters.season': {'$in': seasons}, 'parameters.league': league_id}
        teams = self.find_documents('teams', query)

        LOGGER.info(f'MongoDB: Fetching teams for league {league_id} for seasons {seasons}')
        return teams

    def upsert_many_team_stats(self, team_stats: List[dict]):
        self.bulk_upsert_documents('teams_stats', team_stats)
        LOGGER.info('Upserted team stats data')

    def get_team_stats(self, team_id: str, league_id: str, season: str):
        query = {'parameters.team': team_id, 'parameters.league': league_id, 'parameters.season': season}
        team_stats = self.find_document('teams_stats', query)
        LOGGER.info(f'Fetching stats for team {team_id} for league {league_id} for season {season}')
        return team_stats



import logging
from typing import List, Union

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

logger = logging.getLogger(__name__)


class LineupsRepository(MongoDBBaseRepository):
    DB_NAME = 'api_football'

    def __init__(self):
        super().__init__(self.DB_NAME)

    def upsert_many_lineups(self, lineups: List[dict]):
        self.bulk_upsert_documents('lineups', lineups)
        logger.info('Upserted fixture lineups data')

    def get_lineups(self, fixture_id: str):
        query = {'parameters.fixture': fixture_id}
        lineups = self.find_document('lineups', query)
        logger.info(f'Fetching lineups for fixture {fixture_id}')
        return lineups

    def get_many_lineups(self, fixture_ids: List[str]):
        query = {'parameters.fixture': {'$in': fixture_ids}}
        lineups = self.find_documents('lineups', query)
        logger.info(f'Fetching lineups for fixtures {fixture_ids}')
        return lineups



from src.architecture.models.league import League
from src.architecture.models.team import Team


class Injury:
    def __init__(
            self,
            player_id,
            player_name,
            team,
            fixture_id,
            fixture_date,
            league
    ):
        self.player_id = player_id
        self.player_name = player_name
        self.team = team
        self.fixture_id = fixture_id
        self.fixture_date = fixture_date
        self.league = league

    @classmethod
    def from_dict(cls, response):
        data = response['data']
        return cls(
            player_id=data['player']['id'],
            player_name=data['player']['name'],
            team=Team(
                team_id=data['team']['id'],
                name=data['team']['name'],
            ),
            fixture_id=data['fixture']['id'],
            fixture_date=data['fixture']['date'],
            league=League(
                league_id=data['league']['id'],
                name=data['league']['name'],
                country=data['league']['country']
            )
        )



import logging

from src.models.injury import Injury
from src.repositories.fixture_stats_repository import FixtureStatsRepository
from src.repositories.injury_repository import InjuryRepository
from src.services.fixture_service import FixtureService

LOGGER = logging.getLogger(__name__)


class InjuriesService:
    def __init__(self):
        self.injuries_repository = InjuryRepository()
        self.fixture_service = FixtureService()

    def upsert_many_injuries(self, fixture_injuries):
        self.injuries_repository.bulk_upsert_documents('injuries', fixture_injuries)
        LOGGER.info('Upserted injuries data')

    def upsert_injuries(self, fixture_injuries):
        self.injuries_repository.upsert_document('injuries', fixture_injuries)
        LOGGER.info('Upserted injuries data')

    def get_injuries(self, fixture_id: str):
        response = self.injuries_repository.get_injuries(fixture_id)

        if not response.get('data', []):
            return []

        fixture_stats = [Injury.from_dict(injury) for injury in response]
        return fixture_stats

    def get_team_injuries(self, team_id: str, league_id: str, season: str):
        fixture_ids = self.fixture_service.get_fixture_ids(team_id, league_id, season)
        team_injuries = self.injuries_repository.get_team_injuries(fixture_ids)
        team_injuries = [Injury.from_dict(injuries) for injuries in team_injuries]
        return team_injuries



import os
from pymongo import MongoClient

# Load environment variables or set defaults
MONGO_URI = os.getenv('MONGO_URI', 'mongodb://192.168.0.250:27017/')
#MONGO_URI = os.getenv('MONGO_URI', 'mongodb://localhost:27017/')
#MONGO_DB_NAME = os.getenv('MONGO_DB_NAME', 'local')


class MongoDB:
    def __init__(self, db_name, uri=MONGO_URI):
        self.client = MongoClient(uri)
        self.db = self.client[db_name]

    def get_db(self):
        """
        Return MongoDB instance
        """
        return self.db

    def close(self):
        self.client.close()



from src.models.league import League
from src.models.team import Team


class Fixture:
    def __init__(
            self,
            fixture_id: str,
            datetime,
            status,

            league,
            season,
            league_round,

            home_team: Team,
            ht_winner,

            away_team: Team,
            at_winner,

            fulltime_goals,
            halftime_goals,
    ):
        self.fixture_id = str(fixture_id)
        self.datetime = datetime
        self.status = status

        self.league = str(league)
        self.season = str(season)
        self.league_round = league_round

        self.home_team = home_team
        self.ht_winner = ht_winner

        self.away_team = away_team
        self.at_winner = at_winner

        self.fulltime_goals = fulltime_goals
        self.halftime_goals = halftime_goals

    @classmethod
    def from_dict(cls, data):
        return cls(
            fixture_id=data['fixture']['id'],
            datetime=data['fixture']['date'],
            status=data['fixture']['status'],
            league=League(
                league_id=data['league']['id'],
                name=data['league']['name']
            ),
            season=data['league']['season'],
            league_round=data['league']['round'],
            home_team=Team(
                team_id=data['teams']['home']['id'],
                name=data['teams']['home']['name'],
            ),
            ht_winner=data['teams']['home']['winner'],
            away_team=Team(
                team_id=data['teams']['away']['id'],
                name=data['teams']['away']['name'],
            ),
            at_winner=data['teams']['away']['winner'],
            fulltime_goals=data['score']['fulltime'],
            halftime_goals=data['score']['halftime'],
        )

    def get_fixture_date(self):
        return self.datetime.split('T')[0]



from typing import Union, List

from src.models.team import Team
from src.models.team_stats import TeamStats
from src.repositories.team_repository import TeamRepository


class TeamService:
    def __init__(self):
        self.team_repository = TeamRepository()

    def upsert_many_teams(self, teams: List[dict]):
        self.team_repository.upsert_many_teams(teams)

    def get_teams(self, league_id: str, season: str) -> List[Team]:
        response = self.team_repository.get_teams(league_id, season)

        if not response.get('data', []):
            raise ValueError(f'No teams found for league {league_id} and season {season}')

        teams = [Team.from_dict(team) for team in response['data']]
        return teams

    def get_team_ids(self, league_id: int, seasons: Union[int, List[int]]) -> List[int]:
        teams_over_seasons = self.team_repository.get_many_teams(league_id, seasons)

        team_ids = []
        for teams in teams_over_seasons:
            team_ids.extend([team['team']['id'] for team in teams['data']])

        team_ids = list(map(int, set(team_ids)))
        team_ids.sort()
        return team_ids



from src.models.team import Team


class TeamFixtureStats:
    def __init__(
            self,
            team,

            shots_on_goal,
            shots_off_goal,
            total_shots,
            blocked_shots,
            shots_insidebox,
            shots_outsidebox,
            fouls,
            corner_kicks,
            offsides,
            ball_possession,
            yellow_cards,
            red_cards,
            goalkeeper_saves,
            total_passes,
            pass_accuracy,
            passes,
    ):
        self.team = team

        self.shots_on_goal = shots_on_goal
        self.shots_off_goal = shots_off_goal
        self.total_shots = total_shots
        self.blocked_shots = blocked_shots
        self.shots_insidebox = shots_insidebox
        self.shots_outsidebox = shots_outsidebox
        self.fouls = fouls
        self.corner_kicks = corner_kicks
        self.offsides = offsides
        self.ball_possession = ball_possession  # Percentage
        self.yellow_cards = yellow_cards
        self.red_cards = red_cards
        self.goalkeeper_saves = goalkeeper_saves
        self.total_passes = total_passes
        self.pass_accuracy = pass_accuracy
        self.passes = passes  # Percentage

    @classmethod
    def from_dict(cls, response):
        data = TeamFixtureStats.convert_statistics_to_dict(response)

        return cls(
            team=Team(
                team_id=data['team']['id'],
                name=data['team']['name'],
            ),
            shots_on_goal=data['statistics']['Shots on Goal'],
            shots_off_goal=data['statistics']['Shots off Goal'],
            total_shots=data['statistics']['Total Shots'],
            blocked_shots=data['statistics']['Blocked Shots'],
            shots_insidebox=data['statistics']['Shots insidebox'],
            shots_outsidebox=data['statistics']['Shots outsidebox'],
            fouls=data['statistics']['Fouls'],
            corner_kicks=data['statistics']['Corner Kicks'],
            offsides=data['statistics']['Offsides'],
            ball_possession=data['statistics']['Ball Possession'],
            yellow_cards=data['statistics']['Yellow Cards'],
            red_cards=data['statistics']['Red Cards'],
            goalkeeper_saves=data['statistics']['Goalkeeper Saves'],
            total_passes=data['statistics']['Total passes'],
            pass_accuracy=data['statistics']['Passes accurate'],
            passes=data['statistics']['Passes %'],
        )

    @staticmethod
    def convert_statistics_to_dict(data):
        # Convert the JSON structure to a more accessible dictionary format
        data['statistics'] = {
            stat['type']: stat['value'] for stat in data['statistics']
        }
        return data





class Team:
    def __init__(self, team_id, name):
        self.team_id = team_id
        self.name = name

    @classmethod
    def from_dict(cls, data):
        """
        Create a Team object from a dictionary.
        """
        return cls(
            team_id=data['team']['id'],
            name=data['team']['name'],
        )



from src.models.team_ratings import TeamRatings
from src.repositories.team_ratings_repository import TeamRatingsRepository


class TeamRatingsService:
    def __init__(self):
        self.team_ratings_repository = TeamRatingsRepository()

    def upsert_many_team_ratings(self, team_ratings):
        self.team_ratings_repository.upsert_many_team_ratings(team_ratings)

    def insert_team_ratings(self, team_ratings):
        self.team_ratings_repository.insert_team_ratings(team_ratings)

    def get_team_ratings(self, league_id: str, season: str):
        team_ratings = self.team_ratings_repository.get_team_ratings(league_id, season)
        team_ratings = [TeamRatings.from_dict(response) for response in team_ratings]
        return team_ratings



from src.models.team_lineups import TeamLineups


class Lineups:
    def __init__(
            self,
            fixture_id,
            home_team_injuries,
            away_team_injuries
    ):
        self.fixture_id = fixture_id
        self.home_team_injuries = home_team_injuries
        self.away_team_injuries = away_team_injuries

    @classmethod
    def from_dict(cls, response):
        fixture_id = response['parameters']['fixture']
        data = response['data']

        home_team_injuries = TeamLineups.from_dict(data[0])
        away_team_injuries = TeamLineups.from_dict(data[1])

        return cls(
            fixture_id=fixture_id,
            home_team_injuries=home_team_injuries,
            away_team_injuries=away_team_injuries
        )



from src.models.team import Team


class TeamRatings:
    def __init__(
            self,
            league,
            season,
            team_ratings
    ):
        self.league = league
        self.season = season
        self.team_ratings = team_ratings

    @classmethod
    def from_dict(cls, response):
        return cls(
            league=response['parameters']['league'],
            season=response['parameters']['season'],
            team_ratings=response['data']
        )



import logging

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

logger = logging.getLogger(__name__)


class TeamRatingsRepository(MongoDBBaseRepository):
    DB_NAME = 'sofifa'

    def __init__(self):
        super().__init__(self.DB_NAME)

    def upsert_many_team_ratings(self, odds):
        self.bulk_upsert_documents('team_ratings', odds)
        logger.info('Upsert team ratings data')

    def insert_team_ratings(self, odds):
        self.insert_document('team_ratings', odds)
        logger.info('Inserted team ratings data')

    def get_team_ratings(self, league_id: str, season: str):
        query = {'parameters.league': league_id, 'parameters.season': season}
        team_ratings = self.find_document('odds', query)
        logger.info(f'Fetching odds for league {league_id} for season {season}')
        return team_ratings



from typing import List

from src.models.team_stats import TeamStats
from src.repositories.team_stats_repository import TeamStatsRepository


class TeamStatsService:
    def __init__(self):
        self.team_repository = TeamStatsRepository()

    def upsert_many_team_stats(self, team_stats: List[dict]):
        self.team_repository.upsert_many_team_stats(team_stats)

    def get_team_stats(self, team_id: str, league_id: str, season: str):
        response = self.team_repository.get_team_stats(team_id, league_id, season)

        if not response.get('data', []):
            raise ValueError(f'No team stats found for team {team_id}, league {league_id} and season {season}')

        team_stats = TeamStats.from_dict(response)
        return team_stats



from src.models.odds import Odds
from src.repositories.odds_repository import OddsRepository
from src.services.fixture_service import FixtureService


class OddsService:
    def __init__(self, db_name):
        self.odds_repository = OddsRepository(db_name)
        self.fixture_service = FixtureService()

    def upsert_many_odds(self, odds):
        self.odds_repository.upsert_many_odds(odds)

    def insert_odds(self, odds):
        self.odds_repository.insert_odds(odds)

    def get_team_odds(self, team_id: str, league_id: str, season: str):
        fixture_ids = self.fixture_service.get_fixture_ids(team_id, league_id, season)
        team_odds = self.odds_repository.get_team_odds(fixture_ids)
        team_odds = [Odds.from_dict(response['data']) for response in team_odds]

        return team_odds

    def get_league_odds(self, league_id: str, season: str):
        league_odds = self.odds_repository.get_league_odds(league_id, season)
        league_odds = [Odds.from_dict(response['data']) for response in league_odds]

        return league_odds



from setuptools import setup, find_packages

setup(
    name='buco_db_manager',
    version='0.1',
    packages=find_packages(),
    install_requires=[
        'PyYAML',
        'RapidFuzz',
        'pymongo',
        'dnspython',
        'Unidecode'
    ],
)



from src.models.team import Team


class Odds:
    def __init__(
            self,
            fixture_id: str,
            date,

            home_team: Team,
            away_team: Team,

            result,
            over_under,
            btts,
            dnb,
    ):
        self.fixture_id = fixture_id
        self.date = date

        self.home_team = home_team
        self.away_team = away_team

        self.result = result
        self.over_under = over_under
        self.btts = btts
        self.dnb = dnb

    @classmethod
    def from_dict(cls, data):
        return cls(
            fixture_id=data['fixture']['id'],
            date=data['date'],

            home_team=Team(
                team_id=data['teams']['home']['id'],
                name=data['teams']['home']['name'],
            ),
            away_team=Team(
                team_id=data['teams']['away']['id'],
                name=data['teams']['away']['name'],
            ),

            result=data['odds']['1X2'],
            over_under=data['odds']['over_under'],
            btts=data['odds']['btts'],
            dnb=data['odds']['dnb'],
        )



import logging

from src.mongo_db.mongo_db_repository import MongoDBBaseRepository

logger = logging.getLogger(__name__)


class OddsRepository(MongoDBBaseRepository):

    def __init__(self, db_name):
        super().__init__(db_name)

    def upsert_many_odds(self, odds):
        self.bulk_upsert_documents('odds', odds)
        logger.info('Upsert odds data')

    def insert_odds(self, odds):
        self.insert_document('odds', odds)
        logger.info('Inserted odds data')

    def get_odds(self, fixture_id: str, league_id: str, season: str):
        query = {'parameters.fixture': fixture_id, 'parameters.league': league_id, 'parameters.season': season}
        odds = self.find_document('odds', query)
        logger.info(f'Fetching odds for league {league_id} for season {season}')
        return odds

    def get_team_odds(self, fixture_ids: list):
        query = {'parameters.fixture': {'$in': fixture_ids}}
        odds = self.find_documents('odds', query)
        logger.info(f'Fetching xgoals for fixtures {fixture_ids}')
        return odds

    def get_league_odds(self, league_id: str, season: str):
        query = {'parameters.league': league_id, 'parameters.season': season}
        odds = self.find_documents('odds', query)
        logger.info(f'Fetching odds for league {league_id} for season {season}')
        return odds



from typing import List

from src.models.fixture import Fixture
from src.repositories.fixture_repository import FixtureRepository


class FixtureService:

    def __init__(self):
        self.fixture_repository = FixtureRepository()

    def upsert_many_fixtures(self, fixtures: List[dict]):
        self.fixture_repository.upsert_many_fixtures(fixtures)

    def get_team_fixtures(self, team_id: int, league_id: int, season: int) -> List[Fixture]:
        response = self.fixture_repository.get_team_fixtures(team_id, league_id, season)
        fixtures_data = response.get('data', [])

        if not fixtures_data:
            raise ValueError(f'No fixtures found for team {team_id}, league {league_id} and season {season}')

        fixtures = [Fixture.from_dict(team) for team in fixtures_data]
        fixtures.sort(key=lambda x: x.datetime)
        return fixtures

    def get_league_fixtures(self, league_id: int, season: int) -> List[Fixture]:
        response = self.fixture_repository.get_league_fixtures(league_id, season)
        fixtures_data = [fixture for team in response for fixture in team.get('data', [])]
        fixtures_data = list({fixture['fixture']['id']: fixture for fixture in fixtures_data}.values())

        if not fixtures_data:
            raise ValueError(f'No fixtures found league {league_id} and season {season}')

        fixtures = [Fixture.from_dict(team) for team in fixtures_data]
        fixtures.sort(key=lambda x: (x.datetime, x.fixture_id))
        return fixtures

    def get_fixture_ids(self, team_id: int, league_id: int, season: int) -> list:
        fixtures = self.fixture_repository.get_team_fixtures(team_id, league_id, season)
        fixture_ids = [fixture['fixture']['id'] for fixture in fixtures['data']]
        fixture_ids = list(map(str, fixture_ids))
        fixture_ids.sort()
        return fixture_ids

    def get_league_fixture_ids(self, league_id: int, season: int) -> list:
        fixtures = self.fixture_repository.get_league_fixtures(league_id, season)
        fixture_ids = [fixture['fixture']['id'] for team in fixtures for fixture in team['data']]
        fixture_ids = list(map(str, fixture_ids))
        fixture_ids.sort()
        return fixture_ids

    def get_fixture_dates(self, team_id: int, league_id: int, season: int) -> list:
        fixtures = self.fixture_repository.get_team_fixtures(team_id, league_id, season)
        fixture_dates = [fixture['fixture']['date'] for fixture in fixtures['data']]
        return fixture_dates

    def get_fixture_by_round(self, league_round: str, team_id: int, league_id: int, season: int) -> Fixture:
        league_round = 'Regular Season - ' + league_round
        fixtures = self.fixture_repository.get_team_fixtures(team_id, league_id, season)

        for fixture in fixtures['data']:
            if fixture['league']['round'] == league_round:
                return Fixture.from_dict(fixture)

        raise ValueError(f'No fixture found for round {league_round} | team {team_id} | league {league_id} | season {season}')

    def upsert_many_rounds(self, league_rounds):
        self.fixture_repository.upsert_many_rounds(league_rounds)

    def get_number_of_rounds(self, league_id: int, season: int) -> int:
        league_rounds = self.fixture_repository.get_rounds(league_id, season)
        league_rounds = [league_round for league_round in league_rounds['data'] if 'Regular Season' in league_round]
        return len(league_rounds)



from typing import Dict, Any, List

from pymongo import UpdateOne

from src.mongo_db.mongo_db import MongoDB


class MongoDBBaseRepository(MongoDB):
    def __init__(self, db_name):
        super().__init__(db_name)

    def insert_document(self, collection_name, document):
        collection = self.db[collection_name]
        return collection.insert_one(document).inserted_id

    def insert_many_documents(self, collection_name, documents):
        collection = self.db[collection_name]
        return collection.insert_many(documents).inserted_ids

    def find_document(self, collection_name, query):
        collection = self.db[collection_name]
        return collection.find_one(query)

    def find_documents(self, collection_name: str, query: Dict[str, Any]) -> List[Dict[str, Any]]:
        collection = self.db[collection_name]
        return list(collection.find(query))

    def update_document(self, collection_name, query, update):
        collection = self.db[collection_name]
        return collection.update_one(query, {'$set': update}).modified_count

    def delete_document(self, collection_name, query):
        collection = self.db[collection_name]
        return collection.delete_one(query).deleted_count

    def upsert_document(self, collection_name, item):
        collection = self.db[collection_name]

        query = {'parameters': item['parameters']}
        update = {'$set': item}

        result = collection.update_one(query, update, upsert=True)
        return result.raw_result if result else None

    def bulk_upsert_documents(self, collection_name, data):
        collection = self.db[collection_name]
        operations = []

        for item in data:
            query = {'parameters': item['parameters']}
            update = {'$set': item}
            operations.append(UpdateOne(query, update, upsert=True))

        if operations:
            result = collection.bulk_write(operations)
            return result.bulk_api_result

        return None



from src.models.xgoals import XGoals
from src.repositories.xgoals_repository import XGoalsRepository
from src.services.fixture_service import FixtureService


class XGoalsService:
    def __init__(self, db_name):
        self.xgoals_repository = XGoalsRepository(db_name)
        self.fixture_service = FixtureService()

    def upsert_many_fixture_xg(self, xg):
        self.xgoals_repository.upsert_many_fixture_xg(xg)

    def get_xgoals(self, fixture_id: str):
        xgoals = self.xgoals_repository.get_xgoals(fixture_id)
        return xgoals

    def get_xgoals_over_season(self, team_id: str, league_id: str, season: str):
        fixture_ids = self.fixture_service.get_fixture_ids(team_id, league_id, season)
        xgoals_over_season = self.xgoals_repository.get_many_xgoals(fixture_ids)
        xgoals_over_season = [XGoals.from_dict(response['data']) for response in xgoals_over_season]

        return xgoals_over_season
